import { Inject, Injectable, Logger } from '@nestjs/common';
import { EventEmitter2 } from '@nestjs/event-emitter';
import { AdRepositoryPort } from '../core/application/ports/ad.repository.port';
import { LoggerBase, MessagePublisherPort } from '@mobicoop/ddd-library';
import { PrismaService } from './prisma.service';
import { AD_MESSAGE_PUBLISHER } from '../ad.di-tokens';
import { AdEntity } from '../core/domain/ad.entity';
import { AdMapper } from '../ad.mapper';
import { ExtendedPrismaRepositoryBase } from '@mobicoop/ddd-library/dist/db/prisma-repository.base';
import { Frequency, Role } from '../core/domain/ad.types';
import { Candidate } from '../core/application/types/algorithm.types';
import { AdSelector } from './ad.selector';
import { MatchQuery } from '../core/application/queries/match/match.query';

export type AdBaseModel = {
  uuid: string;
  driver: boolean;
  passenger: boolean;
  frequency: Frequency;
  fromDate: Date;
  toDate: Date;
  seatsProposed: number;
  seatsRequested: number;
  strict: boolean;
  driverDuration?: number;
  driverDistance?: number;
  passengerDuration?: number;
  passengerDistance?: number;
  fwdAzimuth: number;
  backAzimuth: number;
  createdAt: Date;
  updatedAt: Date;
};

export type AdReadModel = AdBaseModel & {
  waypoints: string;
  direction: string;
  schedule: ScheduleItemModel[];
};

export type AdWriteModel = AdBaseModel & {
  schedule: {
    create: ScheduleItemModel[];
  };
};

export type AdUnsupportedWriteModel = {
  waypoints: string;
  direction: string;
};

export type ScheduleItemModel = {
  uuid: string;
  day: number;
  time: Date;
  margin: number;
  createdAt: Date;
  updatedAt: Date;
};

/**
 *  Repository is used for retrieving/saving domain entities
 * */
@Injectable()
export class AdRepository
  extends ExtendedPrismaRepositoryBase<
    AdEntity,
    AdReadModel,
    AdWriteModel,
    AdUnsupportedWriteModel
  >
  implements AdRepositoryPort
{
  constructor(
    prisma: PrismaService,
    mapper: AdMapper,
    eventEmitter: EventEmitter2,
    @Inject(AD_MESSAGE_PUBLISHER)
    protected readonly messagePublisher: MessagePublisherPort,
  ) {
    super(
      prisma.ad,
      prisma,
      mapper,
      eventEmitter,
      new LoggerBase({
        logger: new Logger(AdRepository.name),
        domain: 'matcher',
        messagePublisher,
      }),
    );
  }

  getCandidates = async (query: MatchQuery): Promise<Candidate[]> => {
    // let candidates: Candidate[] = [];
    const sqlQueries: QueryRole[] = [];
    if (query.driver)
      sqlQueries.push({
        query: AdSelector.select(Role.DRIVER, query),
        role: Role.DRIVER,
      });
    if (query.passenger)
      sqlQueries.push({
        query: AdSelector.select(Role.PASSENGER, query),
        role: Role.PASSENGER,
      });
    const results = await Promise.all(
      sqlQueries.map(
        async (queryRole: QueryRole) =>
          ({
            ads: (await this.queryRawUnsafe(queryRole.query)) as AdEntity[],
            role: queryRole.role,
          } as AdsRole),
      ),
    );
    // console.log(results[0].ads);
    return [];
  };
}

type QueryRole = {
  query: string;
  role: Role;
};

type AdsRole = {
  ads: AdEntity[];
  role: Role;
};
